﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.Common;
using System.Reflection;

using Project.BaseFramework.ORM;
using Project.Model;
using Project.DAO;

using Project.BaseFramework.DataProvider;
using System.Diagnostics;
namespace Project.ConsoleTest
{
    class Program
    {
        static void Main(string[] args)
        {
            TestORM();
            TestPaging();

            Console.ReadLine();
        }

        static void TestORM()
        {
            UserDao dao = new UserDao();
            try
            {
                dao.BeginTransaction();
                // add model
                User mUser1 = new User();
                mUser1.ID = "1";
                mUser1.UserName = "Mike1";
                mUser1.UserPwd = "1234";
                mUser1.CreateBy = "System";
                mUser1.CreateDate = DateTime.Now;
                dao.Add(mUser1);

                Console.WriteLine("Create Model Successfully.");
                string cmdText = "SELECT * FROM TUser WHERE ID='1' ORDER BY ID;";
                List<User> list0 = dao.GetListBySQL(cmdText);
                foreach (User model in list0)
                {
                    Console.WriteLine(model.UserName);
                }
                Console.WriteLine("Get List Successfully.");

                //嵌套事务
                UserDao dao2 = new UserDao();
                try
                {
                    dao2.BeginTransaction();
                    //update model
                    User mUser2 = new User();
                    mUser2.ID = "1";
                    mUser2.UserName = "Mike.Jiang";
                    mUser2.UserPwd = null;
                    mUser2.CreateDate = null;
                    dao.Update(mUser2);
                    Console.WriteLine("Update Model Successfully.");

                    User mUser3 = dao.GetModel("1");
                    if (mUser3 != null)
                        Console.WriteLine(mUser3.UserName);

                    Console.WriteLine("Get Model Successfully.");
                    dao2.CommitTransaction();
                }
                catch
                {
                    dao2.RollbackTransaction();
                }

                dao.Delete("1");

                Console.WriteLine("Delete Model Successfully.");
                dao.CommitTransaction();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                dao.RollbackTransaction();
            }
        }

        static void TestPaging()
        {
            UserDao dao = new UserDao();
            // add 100 user
            for (int i = 1; i <= 100; i++)
            {
                User mUser1 = new User();
                mUser1.ID = i.ToString();
                mUser1.UserName = "Mike" + i.ToString();
                mUser1.UserPwd = "1234";
                mUser1.CreateBy = "System";
                mUser1.CreateDate = DateTime.Now;
                dao.Add(mUser1);
            }

            string cmdText = string.Format("SELECT * FROM TUser");
            DataTable table = DBHelper.ExecutePagingDataTable(CommandType.Text, cmdText, 1, 10, "ORDER BY ID");

            foreach (DataRow row in table.Rows)
            {
                Console.WriteLine(row["ID"]);
            }

            Console.WriteLine("Get Datatabe paging successfuly.");

            List<User> list = dao.GetListPaging(cmdText, 2, 10, "ORDER BY ID");
            foreach (User m in list)
            {
                Console.WriteLine(m.ID);
            }

            Console.WriteLine("Get Model list paging successfully.");


            cmdText = "DELETE FROM TUser";
            dao.ExecuteNonQuery(CommandType.Text, cmdText);
        }

        static void TestCreateModel()
        {
            int times = 1000000;
            var watch1 = new Stopwatch();



            watch1.Start();
            for (var i = 0; i < times; i++)
            {
                CreateUserNormal();
            }
            watch1.Stop();
            Console.WriteLine(string.Format("Create user by new: {0}",watch1.Elapsed));

            watch1.Reset();
            watch1.Start();
            for (var i = 0; i < times; i++)
            {
                CreateConvertUser();
            }
            watch1.Stop();
            Console.WriteLine(string.Format("Create user by convert: {0}", watch1.Elapsed));

            watch1.Reset();
            watch1.Start();
            for (var i = 0; i < times; i++)
            {
                TestReflect<User>.CreateUser(); 
            }
            watch1.Stop();
            Console.WriteLine(string.Format("Create user by reflection: {0}", watch1.Elapsed));
        }

        static User CreateUserNormal()
        {
            return new User();
        }

        static User CreateConvertUser()
        {
            User u = new User();
            Project.BaseFramework.ORM.Model m = u;

            u = (User)m;
            return u;
        }
    }

    public class TestReflect<T> where T : Project.BaseFramework.ORM.Model
    {
        public static T CreateUser()
        {
            T u = Activator.CreateInstance<T>();
            return u;
        }
    }
}
